﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioLab.Common;
using BioLab.ImageProcessing;
using System.Drawing;
using BioLab.GUI.Forms;
using System.Windows.Forms;
using BioLab.ImageProcessing.Topology;
using BioLab.DataStructures;
using System.ComponentModel;

namespace PRLab.FEI
{

    [AlgorithmInfo("Calcolo Istogramma", Category = "FEI")]
    public class HistogramBuilder : ImageOperation<Image<byte>, Histogram>
    {
        [AlgorithmParameter]
        [DefaultValue(false)]
        public bool Sqrt { get; set; }

        [AlgorithmParameter]
        [DefaultValue(0)]
        public int SmoothWindowSize { get; set; }

        public HistogramBuilder(Image<byte> inputImage)
        {
            InputImage = inputImage;
        }

        public HistogramBuilder(Image<byte> inputImage, bool sqrt, int smoothWindowSize)
        {
            InputImage = inputImage;
            Sqrt = sqrt;
            SmoothWindowSize = smoothWindowSize;
        }

        public override void Run()
        {
            Result = new Histogram();
            foreach (var p in InputImage)
            {
                Result[p]++;
            }

            if (Sqrt)
            {
                for (int i = 0; i < 256; i++)
                {
                    Result[i] = (int)Math.Round(Math.Sqrt((double)Result[i]));
                }
            }

            if (SmoothWindowSize > 0)
            {
                Histogram histTmp = Result.Clone();
                for (int i = 0; i < 256; i++)
                {
                    int j1 = Math.Max(0, i - SmoothWindowSize);
                    int j2 = Math.Min(255, i + SmoothWindowSize);
                    int n = 0;
                    int sum = 0;
                    for (int j = j1; j <= j2; j++, n++)
                    {
                        sum += histTmp[j];
                    }
                    Result[i] = sum / n;
                }
            }
        }
    }

    [AlgorithmInfo("Binarizzazione con soglia globale", Category = "FEI")]
    public class ImageBinarization : ImageOperation<Image<byte>, Image<byte>>
    {
        [AlgorithmParameter]
        public int Threshold { get; set; }
        private const byte defaultThreshold = 128;

        public ImageBinarization()
        {
            Threshold = defaultThreshold;
        }

        public override void Run()
        {
            Result = new LookupTableTransform<byte>(InputImage, pixel => (byte)(pixel < Threshold ? 0 : 255)).Execute();
        }
    }

    [AlgorithmInfo("Separa canali RGB", Category = "FEI")]
    public class SplitRGB : Algorithm
    {
        [AlgorithmInput]
        public RgbImage<byte> InputImage { get; set; }

        [AlgorithmOutput]
        public Image<byte> Red { get; set; }
        [AlgorithmOutput]
        public Image<byte> Green { get; set; }
        [AlgorithmOutput]
        public Image<byte> Blue { get; set; }

        public override void Run()
        {
            Red = InputImage.RedChannel.Clone();
            Green = InputImage.GreenChannel.Clone();
            Blue = InputImage.BlueChannel.Clone();
        }
    }

    [AlgorithmInfo("Combina canali RGB", Category = "FEI")]
    public class CombineRGB : Algorithm
    {
        [AlgorithmInput]
        public Image<byte> Red { get; set; }
        [AlgorithmInput]
        public Image<byte> Green { get; set; }
        [AlgorithmInput]
        public Image<byte> Blue { get; set; }

        [AlgorithmOutput]
        public RgbImage<byte> Result { get; set; }


        public override void Run()
        {
            Result = new RgbImage<byte>(Red, Green, Blue);
        }
    }


    [AlgorithmInfo("Variazione luminosità RGB", Category = "FEI")]
    public class BrightnessVariationRgb : ImageOperation<RgbImage<byte>, RgbImage<byte>>
    {
        private int variation;

        [AlgorithmParameter]
        public int Variation
        {
            get
            {
                return variation;
            }

            set
            {
                if (value < -100 || value > 100)
                {
                    throw new ArgumentException("La variazione deve essere compresa fra -100 e 100.");
                }
                variation = value;
            }
        }

        public override void Run()
        {
            var op = new BrightnessVariation { Variation = Variation };
            op.InputImage = InputImage.RedChannel; var r = op.Execute();
            op.InputImage = InputImage.GreenChannel; var g = op.Execute();
            op.InputImage = InputImage.BlueChannel; var b = op.Execute();
            Result = new RgbImage<byte>(r, g, b);
        }
    }

    [AlgorithmInfo("Separa canali HSL", Category = "FEI")]
    public class SplitHSL : Algorithm
    {
        [AlgorithmInput]
        public RgbImage<byte> InputImage { get; set; }

        [AlgorithmOutput]
        public Image<byte> Hue { get; set; }
        [AlgorithmOutput]
        public Image<byte> Saturation { get; set; }
        [AlgorithmOutput]
        public Image<byte> Lightness { get; set; }

        public override void Run()
        {
            var hslImage = InputImage.ToByteHslImage();
            Hue = hslImage.HueChannel.Clone();
            Saturation = hslImage.SaturationChannel.Clone();
            Lightness = hslImage.LightnessChannel.Clone();
        }
    }

    [AlgorithmInfo("Combina canali HSL", Category = "FEI")]
    public class CombineHSL : Algorithm
    {
        [AlgorithmInput]
        public Image<byte> Hue { get; set; }
        [AlgorithmInput]
        public Image<byte> Saturation { get; set; }
        [AlgorithmInput]
        public Image<byte> Lightness { get; set; }

        [AlgorithmOutput]
        public RgbImage<byte> Result { get; set; }


        public override void Run()
        {
            Result = new HslImage<byte>(Hue, Saturation, Lightness).ToByteRgbImage();
        }
    }

    [AlgorithmInfo("Colorize", Category = "FEI")]
    public class Colorize : ImageOperation<Image<byte>, RgbImage<byte>>
    {
        [AlgorithmParameter]
        public byte Hue { get; set; }

        [AlgorithmParameter]
        public byte Saturation { get; set; }

        public override void Run()
        {
            Result = new HslImage<byte>(new Image<byte>(InputImage.Width, InputImage.Height, Hue), new Image<byte>(InputImage.Width, InputImage.Height, Saturation), InputImage).ToByteRgbImage();
        }
    }
}
